#!/bin/sh

CURPATH=`pwd`
OUTDIR=
QUIETFLAG=0
ISCOLLECTDUMP=0
ISCLIENTDUMP=0
ISMINICOLLECT=0
UPINSTALLLOCKFILE="/tmp/HostinfoAnalyzeTool-esx.lock"
ROOTUSEREXC="Permission denied, please run as root."
DEFAULTOUTPUTDIR="/var/tmp"

RUN_CONFLICT_EXITCODE=200
NO_ENOUGH_SPACE_EXITCODE=201

ExitWithError()
{
    if [ -f "${UPINSTALLLOCKFILE}" ]
    then
        rm -f "${UPINSTALLLOCKFILE}"
    fi

    exit "$1"
}

CheckSystemType()
{
    OS_TYPE=`uname`
    if [ "${OS_TYPE}" != "VMkernel" ]
    then
        echo "Only support ESX! The OS is ${OS_TYPE}"
        exit 1
    fi
}

CheckOverlapUpgrade()
{
    if [ -f $UPINSTALLLOCKFILE ]
    then
        pid=`cat $UPINSTALLLOCKFILE`
        if [ "${pid}" != "" ]
        then
            PROCOUNT=`ps -c | awk '{print $1}' | grep -c "^[[:blank:]]*${pid}[[:blank:]]*$"`
            if [ $PROCOUNT = 1 ]
            then
                echo "This shell is already running, exit."
                exit $RUN_CONFLICT_EXITCODE
            fi
        fi
    fi

    echo $$ > $UPINSTALLLOCKFILE
}

CheckUser()
{
    if [ `echo $USER` != root ]
    then
        echo $ROOTUSEREXC
        exit 1
    fi
}

usage()
{
	printf "\nUsage : $0 [-s] [-m|-f] "
	printf "\n\t-s\t : Unattended mode, execute this script without manual confirmation."
	printf "\n\t-m\t : Minimal collection mode, operating system information, excluding logs and dump files, is collected. This cannot be used with -f option."
	printf "\n\t-f\t : Full collection mode, operating system information, including logs and dump files, is collected. This cannot be used with -m option."
	printf "\n\n"
}

PrintYesOrNo()
{
    local userInput=""

    echo "$1"

    while [ 1 -eq 1 ]
    do
        echo -n "<Y|N>:"

        read userInput
        if [ "${userInput}" != "yes" ] && [ "${userInput}" != "y" ] && [ "${userInput}" != "no" ] && [ "${userInput}" != "n" ] \
           && [ "${userInput}" != "YES" ] && [ "${userInput}" != "Y" ] && [ "${userInput}" != "NO" ] && [ "${userInput}" != "N" ]
        then
            echo "you select is incorrect[${userInput}]"
            continue
        fi

        if [ "${userInput}" = "yes" ] || [ "${userInput}" = "y" ] || [ "${userInput}" = "YES" ] || [ "${userInput}" = "Y" ]
        then
            return 0
        else
            return 1
        fi
    done

    return 0
}

NotesInformation()
{
    echo  "NOTE: the tool will collect hostInfo as following:
    1) system information
    2) device list
    3) claimrule information
    4) HBA cards information
    5) UltraPath software information
    6) some important files"
    
    if [ ${ISMINICOLLECT} -eq 0 ]
    then
        echo "    7) log information"
        if [ ${ISCOLLECTDUMP} -eq 1 ] || [ ${ISCLIENTDUMP} -eq 1 ]
        then
            echo "    8) dump file"
        fi
    fi     

    if [ ${QUIETFLAG} -eq 0 ]
    then
        PrintYesOrNo "Do you want to collect these information?"
        if [ $? -ne 0 ]
        then
            ExitWithError 0
        fi
    fi
}

GetParam()
{
    while [ $# -gt 0 ]
    do
        param="$1"
        if [ $(echo "${param}"|grep -c "^-[^-]") -eq 1 ]
        then
            paramtmp=${param#-}

            case "${paramtmp}" in
                "s")
                    QUIETFLAG=1
                    ;;
                "m")
                    ISMINICOLLECT=1
                    ;;
                "f")
                    ISCLIENTDUMP=1
                    ;;
                *)
                    echo "Usage Error.[${param}]"
                    usage
                    ExitWithError 0
                    ;;
            esac
        elif [ $(echo "${param}"|grep -c "^--[^-]") -eq 1 ]
        then
            paramtmp=${param#--}
            case "${paramtmp}" in
                *)
                    echo "Usage Error.[${param}]"
                    usage
                    ExitWithError 0
                    ;;
            esac
        else
            echo "Usage Error.[${param}]"
            usage
            ExitWithError 0
        fi
        shift
    done
}

CheckParam()
{
    if [ ${ISMINICOLLECT} -eq 1 ] && [ ${ISCLIENTDUMP} -eq 1 ]
    then
        echo "Full collection mode and minimal collection mode conflict."
        usage
        ExitWithError 1
    fi
    
    if [ ${ISMINICOLLECT} -eq 1 ]
    then
        return 0
    fi
    
    if [ ${ISCLIENTDUMP} -eq 0 ]
    then
        if [ ${QUIETFLAG} -eq 0 ]
        then
            PrintYesOrNo "WARNING: Do you want to collect dump file? If yes,the dump file may be involved in the collection of private data or suspected by customer,please confirm with the customer before the collection."
            if [ $? -ne 0 ]
            then
                ISCOLLECTDUMP=0
            else
                ISCOLLECTDUMP=1
            fi            
        fi  
    else
        ISCOLLECTDUMP=1
    fi
    
    return 0
}

CheckOutDirValid()
{
    local fsInfo=""
    local isMemFs=1
    
    if [ ! -d "$1" ]
    then
        echo "Current directory is not exist,please input a valid directory."
        return 1
    fi
    
    fsInfo=`vmkfstools -P "$1" -h 2>/dev/null`
    if [ $? -ne 0 ]
    then
        echo "Fail to get filesystem infomation of current directory. Please check and modify the path."
        return 1
    fi    
    
    isMemFs=`echo ${fsInfo} | grep -A1 Partitions | grep -c memory`
    if [ ${isMemFs} -ne 0 ]
    then
        echo "Current directory where the host infomation is collected to may occupy system capacity. Please check and modify the path." 
        return 1
    fi
    
    return 0
}

GetOutDir()
{
    isDirValid=0
    echo "Please input the directory for the collected informations. Make sure the space is large enough to save the collect informations, default is the ${DEFAULTOUTPUTDIR}. "

    while [ $isDirValid -eq 0 ]
    do
        echo -n "Please input the directory : "
        read inputdirectory

        if [ "$inputdirectory" = "" ]
        then
            OUTDIR=$DEFAULTOUTPUTDIR
            isDirValid=1
        else
            tmpDir=${inputdirectory}
            
            CheckOutDirValid ${tmpDir}
            if [ $? -eq 0 ]
            then
                if [ `echo "${tmpDir}" | grep -c -E "^[/]"` -eq 0 ]
                then
                    tmpDir=$(cd ${tmpDir};pwd)
                fi
                
                tmpDir=${tmpDir%*/}
                OUTDIR="$tmpDir"
                isDirValid=1
            fi
        fi
    done
    return 0
}

getLowVersionQLogic()
{
    HBADriList=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep "QLogic" |awk '{print $2}' |sort -u`

    for HBADriver in ${HBADriList}
    do
        vmhbaNameList=`esxcfg-scsidevs -a |grep "${HBADriver}" |awk '{print $1}'`
        vmhbaIdList=`ls /proc/scsi/${HBADriver}/`
        for vmhaId in ${vmhbaIdList}
        do
            for vmhbaName in ${vmhbaNameList}
            do
                RET_COUNT=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep -c "Host Device Name ${vmhbaName}$"`
                if [ ${RET_COUNT} -eq 1 ]
                then
                    DriverVerionLn=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Driver version"`
                    DriverVerion=`echo ${DriverVerionLn##*"Driver version "} |awk -F"-" '{print $1}'`
                    FirmwareVersion=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Firmware version" |awk -F"Firmware version " '{print $2}' |awk -F"," '{print $1}' |awk '{print $1}'`
                    hbaMinmodel=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Fibre Channel" |awk -F"Adapter for " '{print $2}' |awk -F":" '{print $1}' |awk -F"-" '{print $1}'`
                    vmhbaIndex=`cat ${HBAInfoFile} |grep "Device: ${vmhbaName}" |grep -v "${vmhbaName}[0-9]" |awk -F"-" '{print $2}'`

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Minmodel: ${hbaMinmodel}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-FirmwareVersion: ${FirmwareVersion}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-DriverVersion: ${DriverVerion}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Vendor: QLogic"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"
                fi
            done
        done
    done
}

getLowVersionEmulex()
{
    HBADriList=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep "Emulex" |awk '{print $2}' |sort -u`

    for HBADriver in ${HBADriList}
    do
        vmhbaNameList=`esxcfg-scsidevs -a |grep "${HBADriver}" |awk '{print $1}'`
        vmhbaIdList=`ls /proc/scsi/${HBADriver}/`
        for vmhaId in ${vmhbaIdList}
        do
            for vmhbaName in ${vmhbaNameList}
            do
                RET_COUNT=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep -c "ESX Adapter: ${vmhbaName}$"`
                if [ ${RET_COUNT} -eq 1 ]
                then
                    DriverVerion=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep "SCSI" |awk -F"SCSI " '{print $2}'`
                    FirmwareVersion=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep "Firmware Version:" |awk -F"Firmware Version: " '{print $2}'`
                    hbaMinmodel=`cat /proc/scsi/${HBADriver}/${vmhaId} |grep "Fibre Channel Adapter" |awk '{print $2}' |awk -F"-" '{print $1}'`
                    vmhbaIndex=`cat ${HBAInfoFile} |grep "Device: ${vmhbaName}" |grep -v "${vmhbaName}[0-9]" |awk -F"-" '{print $2}'`

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Minmodel: ${hbaMinmodel}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-FirmwareVersion: ${FirmwareVersion}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-DriverVersion: ${DriverVerion}"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

                    InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Vendor: Emulex"
                    sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"
                fi
            done
        done
    done
}

getHighVersionQLogic()
{
    vmhbaNameList=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep "QLogic" |awk '{print $1}'`

    for vmhbaName in ${vmhbaNameList}
    do
        RET_COUNT=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -c "Host Device Name ${vmhbaName}$"`
        if [ ${RET_COUNT} -eq 1 ]
        then
            DriverVerionLn=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Driver version"`
            DriverVerion=`echo ${DriverVerionLn##*"Driver version "} |awk -F"-" '{print $1}'`
            FirmwareVersion=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Firmware version" |awk -F"Firmware version " '{print $2}' |awk -F"," '{print $1}'`
            hbaMinmodel=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -B6 "Host Device Name ${vmhbaName}$" |grep "Fibre Channel" |awk -F"Adapter for " '{print $2}' |awk -F":" '{print $1}' |awk -F"-" '{print $1}'`
            vmhbaIndex=`cat ${HBAInfoFile} |grep "Device: ${vmhbaName}" |grep -v "${vmhbaName}[0-9]" |awk -F"-" '{print $2}'`

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Minmodel: ${hbaMinmodel}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-FirmwareVersion: ${FirmwareVersion}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-DriverVersion: ${DriverVerion}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Vendor: QLogic"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"
        fi
    done
}

getHighVersionEmulex()
{
    vmhbaNameList=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep "Emulex" |awk '{print $1}'`

    for vmhbaName in ${vmhbaNameList}
    do
        RET_COUNT=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -c "Key Value Instance:  ${vmhbaName}/"`
        if [ ${RET_COUNT} -eq 1 ]
        then
            DriverVerion=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -A16 "Key Value Instance:  ${vmhbaName}/" |grep "SCSI" |awk -F"SCSI " '{print $2}'`
            FirmwareVersion=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -A16 "Key Value Instance:  ${vmhbaName}/" |grep "FW Version:" |awk '{print $3}'`
            hbaMinmodel=`/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a |grep -A16 "Key Value Instance:  ${vmhbaName}/" |grep "Fibre Channel Adapter" |awk '{print $2}' |awk -F"-" '{print $1}'`
            vmhbaIndex=`cat ${HBAInfoFile} |grep "Device: ${vmhbaName}" |grep -v "${vmhbaName}[0-9]" |awk -F"-" '{print $2}'`

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Minmodel: ${hbaMinmodel}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-FirmwareVersion: ${FirmwareVersion}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-DriverVersion: ${DriverVerion}"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"

            InsertStr="HostBusAdapterInfo-${vmhbaIndex}-Vendor: Emulex"
            sed -i "/HostBusAdapterInfo-${vmhbaIndex}-Driver:/a ${InsertStr}" "${HBAInfoFile}"
        fi
    done
}

getLowVersionHbaInfo()
{
    RET_COUNT=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep -c "QLogic"`
    if [ ${RET_COUNT} -gt 0 ]
    then
        getLowVersionQLogic
    fi

    RET_COUNT=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep -c "Emulex"`
    if [ ${RET_COUNT} -gt 0 ]
    then
        getLowVersionEmulex
    fi
}

getHighVersionHbaInfo()
{
    RET_COUNT=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep -c "QLogic"`
    if [ ${RET_COUNT} -gt 0 ]
    then
        getHighVersionQLogic
    fi

    RET_COUNT=`esxcfg-scsidevs -a |grep "Fibre Channel" |grep -c "Emulex"`
    if [ ${RET_COUNT} -gt 0 ]
    then
        getHighVersionEmulex
    fi
}

InsertHbaVersionInfo()
{
    ESXVerFlag=0
    MainVersion=`esxcli system version get |grep "Version" |awk -F"Version: " '{print $2}'|awk -F"." '{print $1}'`
    MidVersion=`esxcli system version get |grep "Version" |awk -F"Version: " '{print $2}'|awk -F"." '{print $2}'`

    if [ ! -f ${HBAInfoFile} ]
    then
        return
    fi

    echo "Begin to collect hba driver version and firmware version insert to ${HBAInfoFile}."

    if [ ${MainVersion} -gt 5 ]
    then
        ESXVerFlag=1
    fi

    if [ ${MainVersion} -eq 5 ] && [ ${MidVersion} -ge 5 ]
    then
        ESXVerFlag=1
    fi

    if [ ${ESXVerFlag} -eq 0 ]
    then
        getLowVersionHbaInfo
    else
        getHighVersionHbaInfo
    fi
}

GetLogDiskSpace()
{
    local totalLogSpace=0
    
    if [ -d /etc/cim/log ]
    then
        CimLogSize=`du -sm /etc/cim/log/ |awk '{print $1}'`
    else
        CimLogSize=0
    fi

    if [ -d /scratch/log/ ]
    then
        SystemLogSize=`du -sm /scratch/log/ |awk '{print $1}'`
    else
        SystemLogSize=0
    fi

    totalLogSpace=`expr $CimLogSize + $SystemLogSize`
    
    return $totalLogSpace
}

GetDumpDiskSpace()
{
    local DumpFileSize=0
    
    if [ -d /scratch/core ]
    then
        DumpFileSize=`du -sm /scratch/core |awk '{print $1}'`
    else
        DumpFileSize=0
    fi
    
    return $DumpFileSize
}

CheckDiskSpace()
{
    if [ -z "$1" ]
    then
        echo "param error[$1,$2]"
        ExitWithError 1
    fi
    
    local allNeedSpace=$2
    local dumpNeedSpace=0
    local logNeedSpace=0
    
    AvailableSpace=`vmkfstools -P "$1" -h | grep Capacity | awk -F "," '{print $2}'`
    if [ `echo $AvailableSpace | grep -c TB` -eq 1 ] || [ `echo $AvailableSpace | grep -c PB` -eq 1 ]
    then
        return 0
    fi
    
    GBVaule=`echo $AvailableSpace | grep GB`
    if [ "$GBVaule" != "" ]
    then
        TmpFreeSpace=`echo $GBVaule | awk '{print $1}'`
        OutFreeSpace=`echo $TmpFreeSpace 1024 |awk '{print $1*$2}'`
    fi
    
    MBVaule=`echo $AvailableSpace |grep MB`
    if [ "$MBVaule" != "" ]
    then
        OutFreeSpace=`echo $MBVaule | awk '{print $1}'`
    fi
    
    OutFreeSpace=`echo $OutFreeSpace | awk -F "." '{print $1}'`
    
    if [ $ISMINICOLLECT -eq 0 ]
    then
        GetLogDiskSpace
        logNeedSpace=$?
        
        if [ $ISCOLLECTDUMP -eq 1 ]
        then
            GetDumpDiskSpace
            dumpNeedSpace=$?
        fi
    fi

    allNeedSpace=`expr \( $allNeedSpace + $logNeedSpace + $dumpNeedSpace \) \* 2`
    
    if [ -z "${OutFreeSpace}" ] || [ ${OutFreeSpace} -lt ${allNeedSpace} ]
    then
        echo  "more disk free space need[$1,${allNeedSpace}M]"
        ExitWithError $NO_ENOUGH_SPACE_EXITCODE
    fi

    return 0
}

if [ `pwd|grep -c " "` -ne 0 ]
then
    echo -e -n "\033[31m[ERROR:7]\033[0m ";echo "there is space key in the directory[`pwd`].Please run in other directory."
    ExitWithError 1
fi

CheckSystemType
CheckUser
CheckOverlapUpgrade
GetParam $@
CheckParam
NotesInformation

if [ $QUIETFLAG -eq 0 ]
then
    GetOutDir
else
    OUTDIR=$DEFAULTOUTPUTDIR
fi

CheckDiskSpace "$OUTDIR/" 30


HOSTROOTDIR="$OUTDIR"/collectinfo/hostinfo
SYSINFODIR="$OUTDIR"/collectinfo/hostinfo/sysinfo
SYSINFOFILE=sysinfo.txt
HBAINFODIR="$OUTDIR"/collectinfo/hostinfo/hbainfo
HBAINFOFILE=hbainfo.txt
HOSTINFODIR="$OUTDIR"/collectinfo/hostinfo/host
HOSTINFOFILE=hostinfo.txt
LOGINFODIR="$OUTDIR"/collectinfo/hostinfo/log
LOGINFOFILE=loginfo.txt
DUMPDIR="$OUTDIR"/collectinfo/hostinfo/dump

if [ -d "$HOSTROOTDIR" ]
then
    rm -rf $HOSTROOTDIR
fi

if [ ! -d "$SYSINFODIR" ]
then
    mkdir -p $SYSINFODIR
fi

if [ ! -d "$HBAINFODIR" ]
then
    mkdir -p $HBAINFODIR
fi

if [ ! -d "$HOSTINFODIR" ]
then
    mkdir -p $HOSTINFODIR
fi

if [ -d "$OUTDIR"/host ]
then
    cp -rf "$OUTDIR"/host "$OUTDIR"/collectinfo/

    HBAInfoFile="$OUTDIR"/collectinfo/host/HostBusAdapterInfo.txt
    NetworkInfoFile="$OUTDIR"/collectinfo/host/NetworkInfo.txt

    InsertHbaVersionInfo
fi

if [ -d "$OUTDIR"/esx ]
then
    cp -rf "$OUTDIR"/esx "$OUTDIR"/collectinfo/
fi

echo "Begin to collect system information."

echo "-->vmware -lv" >> $SYSINFODIR/$SYSINFOFILE
vmware -lv >> $SYSINFODIR/$SYSINFOFILE 

echo "-->uname -a" >> $SYSINFODIR/$SYSINFOFILE
uname -a >> $SYSINFODIR/$SYSINFOFILE

echo "-->date" >> $SYSINFODIR/$SYSINFOFILE
date >> $SYSINFODIR/$SYSINFOFILE

echo "-->uptime" >> $SYSINFODIR/$SYSINFOFILE
uptime >> $SYSINFODIR/$SYSINFOFILE

echo "-->ulimit -a" >> $SYSINFODIR/$SYSINFOFILE
ulimit -a >> $SYSINFODIR/$SYSINFOFILE

echo "-->env" >> $SYSINFODIR/$SYSINFOFILE
env >> $SYSINFODIR/$SYSINFOFILE

echo "-->ps -C" >> $SYSINFODIR/$SYSINFOFILE
ps -C >> $SYSINFODIR/$SYSINFOFILE

echo "-->ls -l /etc/init.d/*" >> $SYSINFODIR/$SYSINFOFILE
ls -l /etc/init.d/* >> $SYSINFODIR/$SYSINFOFILE

echo "-->df" >> $SYSINFODIR/$SYSINFOFILE
df >> $SYSINFODIR/$SYSINFOFILE

echo "-->df -k" >> $SYSINFODIR/$SYSINFOFILE
df -k >> $SYSINFODIR/$SYSINFOFILE

echo "-->ls -l /vmfs/devices/disks" >> $SYSINFODIR/$SYSINFOFILE
ls -l /vmfs/devices/disks >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcfg-info -a" >> $SYSINFODIR/$SYSINFOFILE
esxcfg-info -a >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcfg-info -w" >> $SYSINFODIR/$SYSINFOFILE
esxcfg-info -w >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcli hardware cpu list" >> $SYSINFODIR/$SYSINFOFILE
esxcli hardware cpu list >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcli hardware cpu global get" >> $SYSINFODIR/$SYSINFOFILE
esxcli hardware cpu global get >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli hardware platform get" >> $SYSINFODIR/$SYSINFOFILE
esxcli hardware platform get >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "esxcli hardware memory get" >> $SYSINFODIR/$SYSINFOFILE
esxcli hardware memory get >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli storage filesystem list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage filesystem list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli storage vmfs extent list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage vmfs extent list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcfg-advcfg -g /UserVars/CIMLogLevel" >> $SYSINFODIR/$SYSINFOFILE
esxcfg-advcfg -g /UserVars/CIMLogLevel >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli storage nmp device list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage nmp device list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli iscsi session list" >> $SYSINFODIR/$SYSINFOFILE
esxcli iscsi session list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli iscsi session connection list" >> $SYSINFODIR/$SYSINFOFILE
esxcli iscsi session connection list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli iscsi software get" >> $SYSINFODIR/$SYSINFOFILE
esxcli iscsi software get >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

echo "-->esxcli iscsi adapter list" >> $SYSINFODIR/$SYSINFOFILE
esxcli iscsi adapter list >> $SYSINFODIR/$SYSINFOFILE 2>>$SYSINFODIR/$SYSINFOFILE

esxcfg-scsidevs -a | grep scsi_vmk | awk '{print $1}' > .scsiadapter
INDEX=1
Adapter=`sed -n ${INDEX}p .scsiadapter`
while [ "$Adapter" != "" ];
do
    echo "-->esxcli iscsi adapter param get -A $Adapter"  >> $SYSINFODIR/$SYSINFOFILE
    esxcli iscsi adapter param get -A $Adapter >> $SYSINFODIR/$SYSINFOFILE
    INDEX=`expr $INDEX + 1`
    Adapter=`sed -n ${INDEX}p .scsiadapter`
done
rm .scsiadapter

echo "-->vmkload_mod -l" >> $SYSINFODIR/$SYSINFOFILE
vmkload_mod -l >> $SYSINFODIR/$SYSINFOFILE

echo "-->vmkload_mod -l | grep scsi" >> $SYSINFODIR/$SYSINFOFILE
vmkload_mod -l | grep scsi >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcfg-nics -l" >> $SYSINFODIR/$SYSINFOFILE
esxcfg-nics -l >> $SYSINFODIR/$SYSINFOFILE

echo "-->esxcli hardware pci list" >> $SYSINFODIR/$SYSINFOFILE
esxcli hardware pci list >> $SYSINFODIR/$SYSINFOFILE

esxcli storage core adapter list | grep fc |awk '{print $2}' > .hbaadapter
INDEX=1
Adapter=`sed -n ${INDEX}p .hbaadapter`
while [ "$Adapter" != "" ];
do
    echo "-->esxcfg-module -i $Adapter"  >> $SYSINFODIR/$SYSINFOFILE
    esxcfg-module -i $Adapter >> $SYSINFODIR/$SYSINFOFILE
    INDEX=`expr $INDEX + 1`
    Adapter=`sed -n ${INDEX}p .hbaadapter`
done
rm .hbaadapter

echo "-->esxcli system version get" >> $SYSINFODIR/$SYSINFOFILE
esxcli system version get >> $SYSINFODIR/$SYSINFOFILE

echo "-->/etc/init.d/hostd status" >> $SYSINFODIR/$SYSINFOFILE
/etc/init.d/hostd status >> $SYSINFODIR/$SYSINFOFILE 2>/dev/null

echo "-->/etc/init.d/sfcbd-watchdog status" >> $SYSINFODIR/$SYSINFOFILE
/etc/init.d/sfcbd-watchdog status >> $SYSINFODIR/$SYSINFOFILE 2>/dev/null

echo "-->esxcli storage core device world list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage core device world list >> $SYSINFODIR/$SYSINFOFILE 2>/dev/null

echo "End to collect system information."

echo "Begin to collect device information."
echo "--> esxcli storage core device list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage core device list >> $SYSINFODIR/$SYSINFOFILE
echo "End to collect device information."

echo "Begin to collect path information."
echo "--> esxcli storage core path list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage core path list >> $SYSINFODIR/$SYSINFOFILE
echo "End to collect path information."

echo "Begin to collect device path information."
echo "--> esxcfg-mpath -b" >> $SYSINFODIR/$SYSINFOFILE
esxcfg-mpath -b >> $SYSINFODIR/$SYSINFOFILE 2>/dev/null
echo "End to collect device path information."

echo "Begin to collect claimrule information."
echo "-->esxcli storage core claimrule list" >> $SYSINFODIR/$SYSINFOFILE
esxcli storage core claimrule list >> $SYSINFODIR/$SYSINFOFILE
echo "End to collect claimrule information."

echo "Begin to collect virtual machine information."
echo "-->esxcli vm process list" >> $SYSINFODIR/$SYSINFOFILE
esxcli vm process list >> $SYSINFODIR/$SYSINFOFILE 2>/dev/null

VMID=`vim-cmd vmsvc/getallvms |grep -E '^[[:blank:]]*[[:digit:]]' |awk '{print $1}'`
for i in ${VMID}
do
    echo "---vim-cmd vmsvc/get.datastores ${i}"  >> $SYSINFODIR/$SYSINFOFILE
    vim-cmd vmsvc/get.datastores ${i} >> $SYSINFODIR/$SYSINFOFILE
done
echo "End to collect virtual machine information."

echo "Begin to collect hba information."
echo "-->esxcfg-scsidevs -a" > $HBAINFODIR/$HBAINFOFILE
esxcfg-scsidevs -a >>  $HBAINFODIR/$HBAINFOFILE
OSVERSION=`vmware -v | awk '{print $3}'`
if [ "$OSVERSION" = "5.0.0" ] || [ "$OSVERSION" = "5.1.0" ]
then
    QLOGICHBA=`ls /proc/scsi | grep qla`
    if [ "$QLOGICHBA" != "" ]
    then
        QLOGICHBAFILE=`ls /proc/scsi/$QLOGICHBA`
        for line in $QLOGICHBAFILE
        do
            echo "-->cat /proc/scsi/$QLOGICHBA/$line" >> $HBAINFODIR/$HBAINFOFILE
            cat /proc/scsi/$QLOGICHBA/$line >> $HBAINFODIR/$HBAINFOFILE
        done
    fi

    EMULEHBA=`ls /proc/scsi | grep lpfc`
    if [ "$EMULEHBA" != "" ]
    then
        EMULEHBAFILE=`ls /proc/scsi/$EMULEHBA`
        for line in $EMULEHBAFILE
        do
            echo "-->cat /proc/scsi/$EMULEHBA/$line" >> $HBAINFODIR/$HBAINFOFILE
            cat /proc/scsi/$EMULEHBA/$line >> $HBAINFODIR/$HBAINFOFILE
        done
    fi

    BROCADEHBA=`ls /proc/scsi | grep bfa`
    if [ "$BROCADEHBA" != "" ]
    then
        BROCADEHBAFILE=`ls /proc/scsi/$BROCADEHBA`
        for line in $BROCADEHBAFILE
        do
            echo "-->cat /proc/scsi/$BROCADEHBA/$line" >> $HBAINFODIR/$HBAINFOFILE
            cat /proc/scsi/$BROCADEHBA/$line >> $HBAINFODIR/$HBAINFOFILE
        done
    fi
else
    echo "-->esxcli storage core adapter list | grep fc"  >> $HBAINFODIR/$HBAINFOFILE
    HBANAME=`esxcli storage core adapter list | grep fc |awk '{print $2}'`
    for line in $HBANAME
    do
        if [ "$TmpLine" = "$line" ]
        then
            continue
        fi

        TmpLine=$line

        echo "-->esxcfg-module -i $line" >> $HBAINFODIR/$HBAINFOFILE
        esxcfg-module -i $line >> $HBAINFODIR/$HBAINFOFILE
    done
    echo "-->/usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a" >> $HBAINFODIR/$HBAINFOFILE
    /usr/lib/vmware/vmkmgmt_keyval/vmkmgmt_keyval -a >> $HBAINFODIR/$HBAINFOFILE
fi

echo "--> esxcli storage san fc stats get" >> $HBAINFODIR/$HBAINFOFILE
esxcli storage san fc stats get >> $HBAINFODIR/$HBAINFOFILE 2>/dev/null

echo "End to collect hba information."

R8C20=803000
UP_VERSION=`esxcli upadm show version |sed -n '1p' | awk -F ': ' '{print $2}' | awk -F "." '{print $1$2$3}'`
UP_VERSION=$(echo ${UP_VERSION})

echo "Begin to collect ultrapath information."
echo "-->esxcli upadm show version" >>  $HOSTINFODIR/$HOSTINFOFILE
esxcli upadm show version >>  $HOSTINFODIR/$HOSTINFOFILE

echo "-->esxcli upadm show diskarray" >>  $HOSTINFODIR/$HOSTINFOFILE
esxcli upadm show diskarray >>  $HOSTINFODIR/$HOSTINFOFILE

if [ "${UP_VERSION}" -ge "${R8C20}" ]
then
    echo "-->esxcli upadm show vlun -t all" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show vlun -t all >>  $HOSTINFODIR/$HOSTINFOFILE
    echo "-->esxcli upadm show alarmenable" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show alarmenable >>  $HOSTINFODIR/$HOSTINFOFILE
    echo "-->esxcli upadm show pathreliabilityenable" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show pathreliabilityenable >>  $HOSTINFODIR/$HOSTINFOFILE

    echo "-->esxcli upadm show vlun detail" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show vlun -t all | grep [0-9][0-9].* | awk '{print $1}' > .vlunid
    INDEX=1
    ID=`sed -n ${INDEX}p .vlunid`
    while [ "$ID" != "" ];
    do
        if [ $ID -le 16383 ];
        then
            esxcli upadm show vlun -l $ID -t all >> $HOSTINFODIR/$HOSTINFOFILE
            INDEX=`expr $INDEX + 1`
            ID=`sed -n ${INDEX}p .vlunid`
        fi
    done
    rm .vlunid
else
    echo "-->esxcli upadm show vlun" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show vlun >>  $HOSTINFODIR/$HOSTINFOFILE
    echo "-->esxcli upadm show vlun detail" >>  $HOSTINFODIR/$HOSTINFOFILE
    esxcli upadm show vlun | grep [0-9][0-9].* | awk '{print $1}' > .vlunid
    INDEX=1
    ID=`sed -n ${INDEX}p .vlunid`
    while [ "$ID" != "" ];
    do
        if [ $ID -le 16383 ];
        then
            esxcli upadm show vlun -l $ID >> $HOSTINFODIR/$HOSTINFOFILE
            INDEX=`expr $INDEX + 1`
            ID=`sed -n ${INDEX}p .vlunid`
        fi
    done
    rm .vlunid
fi

echo "-->esxcli upadm show path" >>  $HOSTINFODIR/$HOSTINFOFILE
esxcli upadm show path >>  $HOSTINFODIR/$HOSTINFOFILE

echo "-->esxcli upadm show path by array" >>  $HOSTINFODIR/$HOSTINFOFILE
esxcli upadm show diskarray | grep -E "^[[:space:]].*[0-9][0-9].*" | awk '{print $1}' > .arrayid
INDEX=1
ID=`sed -n ${INDEX}p .arrayid`
while [ "$ID" != "" ];
do
    if [ $ID -le 16383 ];
    then
        echo "-->esxcli upadm show path -a $ID" >> $HOSTINFODIR/$HOSTINFOFILE
        esxcli upadm show path -a $ID >> $HOSTINFODIR/$HOSTINFOFILE
        INDEX=`expr $INDEX + 1`
        ID=`sed -n ${INDEX}p .arrayid`
    fi
done
rm .arrayid

echo "-->esxcli upadm check status" >>  $HOSTINFODIR/$HOSTINFOFILE
esxcli upadm check status >>  $HOSTINFODIR/$HOSTINFOFILE

echo "-->cp -rf /etc/cim/config/user_cfg.xml" >> $HOSTINFODIR/$HOSTINFOFILE
cp -rf /etc/cim/config/user_cfg.xml $HOSTINFODIR/

echo "-->esxcli software vib list |grep ultrapath" >> $HOSTINFODIR/$HOSTINFOFILE
esxcli software vib list |grep ultrapath >> $HOSTINFODIR/$HOSTINFOFILE

echo "-->esxcli software vib get -n ultrapath.cli" >> $HOSTINFODIR/$HOSTINFOFILE
esxcli software vib get -n ultrapath.cli >> $HOSTINFODIR/$HOSTINFOFILE

echo "-->esxcli software vib get -n ultrapath.cim" >> $HOSTINFODIR/$HOSTINFOFILE
esxcli software vib get -n ultrapath.cim >> $HOSTINFODIR/$HOSTINFOFILE

echo "-->esxcli software vib get -n ultrapath.mpp" >> $HOSTINFODIR/$HOSTINFOFILE
esxcli software vib get -n ultrapath.mpp >> $HOSTINFODIR/$HOSTINFOFILE

echo "End to collect ultrapath information."

if [ ${ISMINICOLLECT} -ne 1 ]
then
    echo "Begin to collect log information."
    if [ ! -d "$LOGINFODIR" ]
    then
        mkdir -p $LOGINFODIR
    fi
    echo "-->cp -rf /etc/cim/log" >> $LOGINFODIR/$LOGINFOFILE
    cp -rf /etc/cim/log/*  $LOGINFODIR/

    echo "-->cp -rf /scratch/log/" >> $LOGINFODIR/$LOGINFOFILE
    cp -rf /scratch/log/*  $LOGINFODIR/
    echo "End to collect log information."
fi


if [ $ISCOLLECTDUMP -eq 1 ]
then
    echo "Begin to collect the dump file."
    echo "-->cp /scratch/core/*" >> $LOGINFODIR/$LOGINFOFILE
    if [ ! -d $DUMPDIR ]
    then
        mkdir -p $DUMPDIR
    fi
    cp -rf /scratch/core/* $DUMPDIR/ 2>/dev/null
    echo "End to collect the dump file."
fi

echo "Begin to compress files,it may take a few minutes"
cd $OUTDIR/collectinfo
curSYSTIME=`date +%Y%m%d%H%M`
TAR_FILE="ESXI-Info-${curSYSTIME}.tar.gz"
tar -zcf $OUTDIR/$TAR_FILE * 2>/dev/null
if [ $? -ne 0 ]
then
    echo "compress files failed."
    rm -rf $OUTDIR/collectinfo
    ExitWithError 1
fi

cd "$CURPATH"
rm -rf $SYSINFODIR
rm -rf $HBAINFODIR
rm -rf $HOSTINFODIR
rm -rf $LOGINFODIR
rm -rf $DUMPDIR
rm -rf $OUTDIR/collectinfo

echo "Collect hostinfo successfully,the HostInfo is in $OUTDIR/$TAR_FILE."

ExitWithError 0
